@@ -1,10 +1,9 @@
+from importlib import import_module
 from funcy import memoize, merge
 
 from django.conf import settings as base_settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.signals import setting_changed
-from django.db import models
-from django.utils.module_loading import import_string
 
 
 ALL_OPS = {'get', 'fetch', 'count', 'aggregate', 'exists'}
@@ -22,8 +21,9 @@ class Defaults:
     CACHEOPS_SENTINEL = {}
     # NOTE: we don't use this fields in invalidator conditions since their values could be very long
     #       and one should not filter by their equality anyway.
-    CACHEOPS_SKIP_FIELDS = models.FileField, models.TextField, models.BinaryField
+    CACHEOPS_SKIP_FIELDS = "FileField", "TextField", "BinaryField", "JSONField"
     CACHEOPS_LONG_DISJUNCTION = 8
+    CACHEOPS_SERIALIZER = 'pickle'
 
     FILE_CACHE_DIR = '/tmp/cacheops_file_cache'
     FILE_CACHE_TIMEOUT = 60*60*24*30
@@ -32,8 +32,13 @@ class Defaults:
 class Settings(object):
     def __getattr__(self, name):
         res = getattr(base_settings, name, getattr(Defaults, name))
-        if name == 'CACHEOPS_PREFIX':
-            res = res if callable(res) else import_string(res)
+        if name in ['CACHEOPS_PREFIX', 'CACHEOPS_SERIALIZER']:
+            res = import_string(res) if isinstance(res, str) else res
+
+        # Convert old list of classes to list of strings
+        if name == 'CACHEOPS_SKIP_FIELDS':
+            res = [f if isinstance(f, str) else f.get_internal_type(res) for f in res]
+
         # Save to dict to speed up next access, __getattr__ won't be called
         self.__dict__[name] = res
         return res
@@ -42,6 +47,14 @@ settings = Settings()
 setting_changed.connect(lambda setting, **kw: settings.__dict__.pop(setting, None), weak=False)
 
 
+def import_string(path):
+    if "." in path:
+        module, attr = path.rsplit(".", 1)
+        return getattr(import_module(module), attr)
+    else:
+        return import_module(path)
+
+
 @memoize
 def prepare_profiles():
     """
