@@ -1,11 +1,10 @@
-from __future__ import absolute_import
 import warnings
 from contextlib import contextmanager
-import six
 
 from django.core.exceptions import ImproperlyConfigured
+from django.utils.module_loading import import_string
 
-from funcy import decorator, identity, memoize, LazyObject
+from funcy import decorator, identity, memoize, omit, LazyObject
 import redis
 from redis.sentinel import Sentinel
 from .conf import settings
@@ -84,23 +83,28 @@ def redis_client():
     if settings.CACHEOPS_REDIS and settings.CACHEOPS_SENTINEL:
         raise ImproperlyConfigured("CACHEOPS_REDIS and CACHEOPS_SENTINEL are mutually exclusive")
 
+    client_class = CacheopsRedis
+    if settings.CACHEOPS_CLIENT_CLASS:
+        client_class = import_string(settings.CACHEOPS_CLIENT_CLASS)
+
     if settings.CACHEOPS_SENTINEL:
         if not {'locations', 'service_name'} <= set(settings.CACHEOPS_SENTINEL):
             raise ImproperlyConfigured("Specify locations and service_name for CACHEOPS_SENTINEL")
 
-        sentinel = Sentinel(settings.CACHEOPS_SENTINEL['locations'])
+        sentinel = Sentinel(
+            settings.CACHEOPS_SENTINEL['locations'],
+            **omit(settings.CACHEOPS_SENTINEL, ('locations', 'service_name', 'db')))
         return sentinel.master_for(
             settings.CACHEOPS_SENTINEL['service_name'],
-            redis_class=CacheopsRedis,
-            db=settings.CACHEOPS_SENTINEL.get('db', 0),
-            socket_timeout=settings.CACHEOPS_SENTINEL.get('socket_timeout')
+            redis_class=client_class,
+            db=settings.CACHEOPS_SENTINEL.get('db', 0)
         )
 
     # Allow client connection settings to be specified by a URL.
-    if isinstance(settings.CACHEOPS_REDIS, six.string_types):
-        return CacheopsRedis.from_url(settings.CACHEOPS_REDIS)
+    if isinstance(settings.CACHEOPS_REDIS, str):
+        return client_class.from_url(settings.CACHEOPS_REDIS)
     else:
-        return CacheopsRedis(**settings.CACHEOPS_REDIS)
+        return client_class(**settings.CACHEOPS_REDIS)
 
 
 ### Lua script loader
